#!/bin/bash

rm libresources.so

gcc --std=gnu23 -g -fPIC -shared "${1:-resources.c}" -I. -o libresources.so || { echo "DOES NOT COMPILE"; exit; }

make || { echo "SEEMINGLY INCOMPATIBLE"; exit; }

#                   limits:
ulimit -m 10240   # 10MB of the main memory
ulimit -s 64      # 64kB of stack

valgrind_grep(){
    grep 'lost: .* [1-9]' | tr -s ' ' | cut -d ' ' -f 2- | awk '{print "    " $0}'
}

# check if the shipped example file works well
echo -n "example.c output:   "
diff <(sort example.out) <(timeout -k 0.6s 1s ./example 2>/dev/null | sort) && echo ok || echo FAIL

X=$(mktemp -u)
mkfifo "$X"
exec 3<>"$X"
rm "$X"

# all else
run() {
    printf "%-20s" $@
    OUTPUT=$(
        timeout -k ${TO:-0.6}s $(qalc -b10 -t "${TO:-0.6}*3")s valgrind --error-exitcode=13 "$@" 2> >(valgrind_grep>&3)
        RV=$?
        case "$RV" in
            124) echo TIMEOUT;;
            137) echo TIMEOUT+KILL;;
            13) echo MEMCHECK;;
            0) ;;
            *) echo FAIL "($( ((RV>128 && RV<=192)) && kill -l $((RV-128)) || echo $RV ))";;
        esac
    )
    echo "$OUTPUT"
    while read -t 0.01 <&3
    do
        echo "                   $REPLY" >&2
    done
}

echo "valgrind example.c: "
run ./example >/dev/null

run ./01_hello_world
run ./02_err_create
run ./03_err_take
run ./04_err_return
run ./05_create_nums
run ./06_disjoint
run ./07_simple_mutex
run ./08_rereorder
run ./09_interleaved
run ./10_permutes
run ./11_permutes2
TO=2 run ./12_borrow
run ./13_takemany
TO=2 run ./14_order
TO=2 run ./15_dorder

